@@ -121,17 +121,21 @@ class QValuePolicy(object):
 
         # Special case masked out actions (q_value ~= -inf) so that we don't
         # even consider them for exploration.
-        random_valid_action_logits = tf.where(
-            tf.equal(q_values, tf.float32.min),
-            tf.ones_like(q_values) * tf.float32.min, tf.ones_like(q_values))
+        # random_valid_action_logits = tf.where(
+        #     tf.equal(q_values, tf.float32.min),
+        #     tf.ones_like(q_values) * tf.float32.min, tf.ones_like(q_values))
+        random_valid_action_logits = q_values
         random_actions = tf.squeeze(
             tf.multinomial(random_valid_action_logits, 1), axis=1)
 
+        # random_actions = tf.squeeze(
+        #     tf.multinomial([[0.0]], 1), axis=1)
         chose_random = tf.random_uniform(
             tf.stack([batch_size]), minval=0, maxval=1,
             dtype=tf.float32) < cur_epsilon
         self.action = tf.where(chose_random, random_actions,
                                deterministic_actions)
+        # self.action = random_actions
         self.action_prob = None
 
 
@@ -185,6 +189,7 @@ def postprocess_trajectory(policy,
 
 
 def build_q_model(policy, obs_space, action_space, config):
+    print(222222)
     if not isinstance(action_space, Discrete):
         raise UnsupportedSpaceException(
             "Action space {} is not supported for DQN.".format(action_space))
@@ -235,6 +240,7 @@ def build_q_model(policy, obs_space, action_space, config):
 
 def build_q_networks(policy, q_model, input_dict, obs_space, action_space,
                      config):
+    print(1111111)
     # Action Q network
     q_values, q_logits, q_dist = _compute_q_values(
         policy, q_model, input_dict[SampleBatch.CUR_OBS], obs_space,
@@ -299,6 +305,7 @@ def _build_parameter_noise(policy, pnet_params):
 
 
 def build_q_losses(policy, model, _, train_batch):
+    print(33333)
     config = policy.config
     # q network evaluation
     q_t, q_logits_t, q_dist_t = _compute_q_values(
@@ -369,6 +376,7 @@ def clip_gradients(policy, optimizer, loss):
 
 
 def build_q_stats(policy, batch):
+    print(4444444)
     return dict({
         "cur_lr": tf.cast(policy.cur_lr, tf.float64),
     }, **policy.q_loss.stats)
@@ -388,8 +396,9 @@ def setup_late_mixins(policy, obs_space, action_space, config):
 
 
 def _compute_q_values(policy, model, obs, obs_space, action_space):
-    # action_mask = tf.cast(obs[:, 0:300], tf.float32)
-    action_mask = tf.cast(obs[:, 0:100], tf.float32)
+    action_mask = tf.cast(obs[:, 0:500], tf.float32)
+    obs = tf.cast(obs[:, :], tf.float32)
+    # action_mask = tf.cast(obs[:, 0:100], tf.float32)
     config = policy.config
     model_out, state = model({
         "obs": obs,
@@ -433,7 +442,8 @@ def _compute_q_values(policy, model, obs, obs_space, action_space):
             # value = state_score + action_scores_centered + inf_mask
 
             value = state_score + action_scores_centered
-            paddings = tf.ones_like(value) * (-2 ** 32 + 1)
+            paddings = tf.ones_like(value) * (-2 ** 32 + 1.0)
+            # paddings = tf.ones_like(value) * tf.float32.min
             value = tf.where(tf.equal(mask, 1.0), value, paddings)
     else:
         value = action_scores
