@@ -277,19 +277,30 @@ def ptn_rnn_decoder(cell,
                                              tf.ones([batch_size, seq_length], dtype=tf.bool),
                                              tf.tile(tf.math.logical_not(taohua_mask)[tf.newaxis, :], [batch_size, 1]))
                 taohua_ajust_mask = tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.5)
-
             elif mode == "GREEDY":
                 sample_fn = greedy_sample_from_logits
 
-                if_ssr_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.1),
+                if_ssr_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.5),
                                        tf.ones([batch_size, seq_length], dtype=tf.bool),
                                        tf.tile(tf.math.logical_not(l0_ssr_mask)[tf.newaxis, :], [batch_size, 1]))
-                if_taohua_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.9),
+                if_taohua_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.8),
                                           tf.ones([batch_size, seq_length], dtype=tf.bool),
                                           tf.tile(tf.math.logical_not(taohua_mask)[tf.newaxis, :], [batch_size, 1]))
-                if_l2_taohua_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.5),
+                if_l2_taohua_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.2),
                                              tf.ones([batch_size, seq_length], dtype=tf.bool),
                                              tf.tile(tf.math.logical_not(taohua_mask)[tf.newaxis, :], [batch_size, 1]))
+                taohua_ajust_mask = tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.5)
+                # sample_fn = greedy_sample_from_logits
+                #
+                # if_ssr_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.1),
+                #                        tf.ones([batch_size, seq_length], dtype=tf.bool),
+                #                        tf.tile(tf.math.logical_not(l0_ssr_mask)[tf.newaxis, :], [batch_size, 1]))
+                # if_taohua_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.9),
+                #                           tf.ones([batch_size, seq_length], dtype=tf.bool),
+                #                           tf.tile(tf.math.logical_not(taohua_mask)[tf.newaxis, :], [batch_size, 1]))
+                # if_l2_taohua_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.5),
+                #                              tf.ones([batch_size, seq_length], dtype=tf.bool),
+                #                              tf.tile(tf.math.logical_not(taohua_mask)[tf.newaxis, :], [batch_size, 1]))
             elif mode == 'CUSTOM':
                 sample_fn = random_sample_from_logits
                 # sample_fn = custom_sample_from_logits
@@ -426,6 +437,17 @@ def ptn_rnn_decoder(cell,
 
             return tf.stack(output_logits, axis=1), state
         elif mode == "BEAMSEARCH":
+            if_ssr_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.5),
+                                   tf.ones([batch_size, seq_length], dtype=tf.bool),
+                                   tf.tile(tf.math.logical_not(l0_ssr_mask)[tf.newaxis, :], [batch_size, 1]))
+            if_taohua_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.8),
+                                      tf.ones([batch_size, seq_length], dtype=tf.bool),
+                                      tf.tile(tf.math.logical_not(taohua_mask)[tf.newaxis, :], [batch_size, 1]))
+            if_l2_taohua_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.2),
+                                         tf.ones([batch_size, seq_length], dtype=tf.bool),
+                                         tf.tile(tf.math.logical_not(taohua_mask)[tf.newaxis, :], [batch_size, 1]))
+            taohua_ajust_mask = tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.5)
+
             index_matrix_to_beampairs = index_matrix_to_pairs_fn(batch_size, beam_size)
 
             def top_k(acum_logits, logits):
@@ -478,34 +500,28 @@ def ptn_rnn_decoder(cell,
                              for output_ta_flat in list(zip(*outputs_ta_flat))]
                 return outputs_l
 
-            def beam_sample(accum_logits, logits, point_mask, state, pre_output_idxs, i):
+            def beam_sample(accum_logits, logits, point_mask, selected_mask, state, pre_output_idxs, i):
+
                 # sample top_k, last_bema_id:[batch,beam_size], output_idx:[batch,beam_size]
                 accum_logits, last_beam_id, output_idx = top_k(accum_logits,
                                                                logits)  # [batch, beam_size], 前面那个beam path, 后面哪个节点
                 state = beam_select(state, last_beam_id)
                 point_mask = beam_select(point_mask, last_beam_id)
+                selected_mask = beam_select(selected_mask, last_beam_id)
                 output_idx = tf.unstack(output_idx, axis=1)  # [batch] * beam_size
 
-                # onehot = tf.one_hot(output_idx, depth=seq_length, on_value=False, off_value=True, dtype=tf.bool)
-                # selected_mask = tf.math.logical_and(selected_mask, onehot)
-
-                if i + 1 < 3:
-                    l_mask = l1_mask
-                    t_mask = t0_mask
-                elif i + 1 < 6:
-                    l_mask = l2_mask
-                    t_mask = t0_mask
-                    if i + 1 == 5:
-                        t_mask = tf.math.logical_or(t1_mask, t2_mask)
-                else:
-                    l_mask = l3_mask
-                    t_mask = t0_mask
-                    if i + 1 == 8:
-                        t_mask = t1_mask
-                point_mask = [tf.math.logical_and(tf.math.logical_and(l_mask, t_mask),
-                                                  tf.math.logical_and(selected_mask,
-                                                                      tf.one_hot(output_idx[j], depth=seq_length, on_value=False, off_value=True, dtype=tf.bool)))
-                              for j in range(beam_size)]
+                # point_mask_new = []
+                # for j in range(len(point_mask)):
+                #     output_idx_ = output_idx[j]
+                #     point_mask_ = point_mask[j]
+                #     onehot = tf.one_hot(output_idx_, depth=seq_length, on_value=False, off_value=True, dtype=tf.bool)
+                #     point_mask_ = tf.math.logical_and(point_mask_, onehot)
+                #     point_mask_new.append(point_mask_)
+                # point_mask = point_mask_new
+                selected_mask = [
+                    tf.math.logical_and(selected_mask[j],
+                                        tf.one_hot(output_idx[j], depth=seq_length, on_value=False, off_value=True, dtype=tf.bool))
+                    for j in range(beam_size)]
                 # point_mask = [update_mask(output_idx[j], point_mask[j], l_mask + t_mask) for j in range(beam_size)]
 
                 l_output_idx = [tf.expand_dims(t, axis=1)  # [batch, 1] * beam_size
@@ -515,25 +531,89 @@ def ptn_rnn_decoder(cell,
                     output_idxs = list(map(lambda ts: tf.concat(ts, axis=1), zip(pre_output_idxs, l_output_idx)))
                 else:
                     output_idxs = l_output_idx
-                return accum_logits, point_mask, state, output_idx, output_idxs
+                return accum_logits, point_mask, selected_mask, state, output_idx, output_idxs
 
             # initial setting
             state = [state] * beam_size  # [batch, state_size] * beam_size
             point_mask = [point_mask] * beam_size  # [batch, data_len] * beam_size
+            selected_mask = [selected_mask] * beam_size
             # logits -> log pi
             logits = logits - tf.reduce_logsumexp(logits, axis=1, keep_dims=True)
             logits = [logits] * beam_size  # [batch, data_len] * beam_size
             accum_logits = [tf.zeros([batch_size])] * beam_size
 
-            accum_logits, point_mask, state, output_idx, output_idxs = \
-                beam_sample(accum_logits, logits, point_mask, state, None, 0)
+            accum_logits, point_mask, selected_mask, state, output_idx, output_idxs = \
+                beam_sample(accum_logits, logits, point_mask, selected_mask, state, None, 0)
             for i in range(1, res_length):
+                point_mask_new = []
+                for j in range(beam_size):
+                    point_mask_ = point_mask[j]
+                    output_idx_ = output_idx[j]
+                    selected_mask_ = selected_mask[j]
+                    # onehot = tf.one_hot(output_idx_, depth=seq_length, on_value=False, off_value=True, dtype=tf.bool)
+                    # selected_mask_ = tf.math.logical_and(selected_mask_, onehot)
+
+                    if i == 1:
+                        l_mask = l1_mask
+                        t_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.65),
+                                          tf.tile(t2_mask[tf.newaxis, :], [batch_size, 1]),
+                                          tf.tile(t4_mask[tf.newaxis, :], [batch_size, 1]))
+                    elif i == 2:
+                        l_mask = l1_mask
+                        t_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.65),
+                                          tf.tile(t3_mask[tf.newaxis, :], [batch_size, 1]),
+                                          tf.tile(t4_mask[tf.newaxis, :], [batch_size, 1]))
+                    elif i == 3:
+                        l_mask = l2_mask
+                        t_mask = t1_mask
+                    elif i == 4:
+                        l_mask = l2_mask
+                        t_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.65),
+                                          tf.tile(t2_mask[tf.newaxis, :], [batch_size, 1]),
+                                          tf.tile(t4_mask[tf.newaxis, :], [batch_size, 1]))
+                    elif i == 5:
+                        l_mask = l2_mask
+                        t_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.65),
+                                          tf.tile(t3_mask[tf.newaxis, :], [batch_size, 1]),
+                                          tf.tile(t4_mask[tf.newaxis, :], [batch_size, 1]))
+                    elif i == 6:
+                        l_mask = l3_mask
+                        t_mask = t1_mask
+                    elif i == 7:
+                        l_mask = l3_mask
+                        t_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.8),
+                                          tf.tile(t2_mask[tf.newaxis, :], [batch_size, 1]),
+                                          tf.tile(t4_mask[tf.newaxis, :], [batch_size, 1]))
+                    elif i == 8:
+                        l_mask = l3_mask
+                        t_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.65),
+                                          tf.tile(t3_mask[tf.newaxis, :], [batch_size, 1]),
+                                          tf.tile(t4_mask[tf.newaxis, :], [batch_size, 1]))
+                    else:
+                        raise Exception('update point_mask failed, unexcepted i')
+
+                    point_mask_ = tf.math.logical_and(tf.math.logical_and(l_mask, t_mask), selected_mask_)
+                    # endregion
+                    ssr_mask = tf.math.logical_not(tf.math.logical_and(tf.reduce_any(tf.math.logical_and(l0_ssr_mask, tf.math.logical_not(selected_mask_)), axis=1, keep_dims=True), l0_ssr_mask))
+                    th_mask = tf.math.logical_not(tf.math.logical_and(tf.reduce_any(tf.math.logical_and(taohua_mask, tf.math.logical_not(selected_mask_)), axis=1, keep_dims=True), taohua_mask))
+                    if i in [6]:
+                        th_mask = tf.where(taohua_ajust_mask,
+                                           tf.tile(tf.math.logical_not(taohua_mask)[tf.newaxis, :], [batch_size, 1]),
+                                           th_mask)
+                    point_mask_ = tf.math.logical_and(point_mask_, tf.math.logical_and(th_mask, ssr_mask))
+                    point_mask_ = tf.math.logical_and(point_mask_, tf.math.logical_and(if_ssr_mask, if_taohua_mask))
+
+                    if i in [3, 4, 5]:
+                        point_mask_ = tf.math.logical_and(point_mask_, if_l2_taohua_mask)
+                    point_mask_new.append(point_mask_)
+                point_mask = point_mask_new
+
                 logits, state = zip(*[call_cell(output_idx[ik], state[ik], point_mask[ik])  # [batch_size, data_len]
                                       for ik in range(beam_size)])
                 # logits -> log pi
                 logits = [logit_ - tf.reduce_logsumexp(logit_, axis=1, keep_dims=True) for logit_ in logits]
-                accum_logits, point_mask, state, output_idx, output_idxs = \
-                    beam_sample(accum_logits, logits, point_mask, state, output_idxs, i)
+                accum_logits, point_mask, selected_mask, state, output_idx, output_idxs = \
+                    beam_sample(accum_logits, logits, point_mask, selected_mask, state, output_idxs, i)
             return accum_logits[0], output_idxs[0], state[0]
         else:
             raise NotImplementedError("unknown mode: %s. Available modes: [SAMPLE, TRAIN, GREEDY, BEAMSEARCH]" % mode)
