@@ -46,7 +46,7 @@ def read_tfrecord(filename, config, is_pred=False, batch_size=32, random_read=Tr
         }
 
         for seq_num_i in range(config['seq_num']):
-            context_features['sequence_id_' + str(seq_num_i)] = tf.io.VarLenFeature(dtype=tf.int64)
+            context_features['sequence_id_' + str(seq_num_i)] = tf.io.VarLenFeature(dtype=tf.float32)
             context_features['sequence_time_' + str(seq_num_i)] = tf.io.VarLenFeature(dtype=tf.int64)
             context_features['sequence_time_gaps_' + str(seq_num_i)] = tf.io.VarLenFeature(dtype=tf.int64)
 
@@ -130,7 +130,7 @@ def read_tfrecord(filename, config, is_pred=False, batch_size=32, random_read=Tr
     return dataset_train
 
 
-def get_tfrecord_dataloader(file, config, batch_size=None, random_read=None):
+def get_tfrecord_dataloader(file, config, is_pred=False, batch_size=None, random_read=None):
     """Get the DataLoader used to read TFRecordDataset.
 
     With the Dataloader, you can read TFRecordDataset concurrently.
@@ -154,7 +154,7 @@ def get_tfrecord_dataloader(file, config, batch_size=None, random_read=None):
     batch_size = param_check(config, 'batchsize', batch_size, 32)
     random_read = param_check(config, 'random_read', random_read, True)
 
-    data_loader = read_tfrecord(file, config, batch_size=batch_size, random_read=random_read)
+    data_loader = read_tfrecord(file, config, is_pred=is_pred, batch_size=batch_size, random_read=random_read)
 
     return data_loader
 
